@@ -90,7 +90,7 @@ send_response_in_thread_func (GTask        *task,
   if (response != 0)
     goto out;
 
-  if (!g_variant_lookup (options, "b", "writable", &writable))
+  if (!g_variant_lookup (options, "writable", "b",  &writable))
     writable = FALSE;
 
   choices = g_variant_lookup_value (options, "choices", G_VARIANT_TYPE ("a(ss)"));
@@ -224,7 +224,6 @@ check_filter (GVariant *filter,
         }
       else if (type == 1)
         {
-        }
           /* TODO: validate content type */
           if (string[0] == 0)
             {
@@ -234,6 +233,7 @@ check_filter (GVariant *filter,
                                    "invalid content type");
               return FALSE;
             }
+        }
       else
         {
           g_set_error (error,
@@ -561,6 +561,7 @@ file_chooser_iface_init (XdpFileChooserIface *iface)
 static void
 file_chooser_init (FileChooser *fc)
 {
+  xdp_file_chooser_set_version (XDP_FILE_CHOOSER (fc), 1);
 }
 
 static void
